@@ -23,12 +23,15 @@ namespace Colors {
     }
 
 // globals for testing
-bool                          barrelRoll  = false;
-std::string                   flagsFound  = "";
-Hyprlang::CConfig*            pConfig     = nullptr;
-std::string                   currentPath = "";
+bool                            barrelRoll    = false;
+std::string                     flagsFound    = "";
+Hyprlang::CConfig*              pConfig       = nullptr;
+std::string                     currentPath   = "";
+std::string                     ignoreKeyword = "";
+std::string                     useKeyword    = "";
+static std::vector<std::string> categoryKeywordActualValues;
 
-static Hyprlang::CParseResult handleDoABarrelRoll(const char* COMMAND, const char* VALUE) {
+static Hyprlang::CParseResult   handleDoABarrelRoll(const char* COMMAND, const char* VALUE) {
     if (std::string(VALUE) == "woohoo, some, params")
         barrelRoll = true;
 
@@ -44,6 +47,28 @@ static Hyprlang::CParseResult handleFlagsTest(const char* COMMAND, const char* V
     return result;
 }
 
+static Hyprlang::CParseResult handleCategoryKeyword(const char* COMMAND, const char* VALUE) {
+    categoryKeywordActualValues.push_back(VALUE);
+
+    return Hyprlang::CParseResult();
+}
+
+static Hyprlang::CParseResult handleTestIgnoreKeyword(const char* COMMAND, const char* VALUE) {
+    ignoreKeyword = VALUE;
+
+    return Hyprlang::CParseResult();
+}
+
+static Hyprlang::CParseResult handleTestUseKeyword(const char* COMMAND, const char* VALUE) {
+    useKeyword = VALUE;
+
+    return Hyprlang::CParseResult();
+}
+
+static Hyprlang::CParseResult handleNoop(const char* COMMAND, const char* VALUE) {
+    return Hyprlang::CParseResult();
+}
+
 static Hyprlang::CParseResult handleSource(const char* COMMAND, const char* VALUE) {
     std::string PATH = std::filesystem::canonical(currentPath + "/" + VALUE);
     return pConfig->parseFile(PATH.c_str());
@@ -88,12 +113,14 @@ int main(int argc, char** argv, char** envp) {
         config.addConfigValue("testStringColon", "");
         config.addConfigValue("testEnv", "");
         config.addConfigValue("testVar", (Hyprlang::INT)0);
+        config.addConfigValue("categoryKeyword", (Hyprlang::STRING) "");
         config.addConfigValue("testStringQuotes", "");
         config.addConfigValue("testStringRecursive", "");
         config.addConfigValue("testCategory:testValueInt", (Hyprlang::INT)0);
         config.addConfigValue("testCategory:testValueHex", (Hyprlang::INT)0xA);
         config.addConfigValue("testCategory:nested1:testValueNest", (Hyprlang::INT)0);
         config.addConfigValue("testCategory:nested1:nested2:testValueNest", (Hyprlang::INT)0);
+        config.addConfigValue("testCategory:nested1:categoryKeyword", (Hyprlang::STRING) "");
         config.addConfigValue("testDefault", (Hyprlang::INT)123);
         config.addConfigValue("testCategory:testColor1", (Hyprlang::INT)0);
         config.addConfigValue("testCategory:testColor2", (Hyprlang::INT)0);
@@ -107,6 +134,10 @@ int main(int argc, char** argv, char** envp) {
         config.registerHandler(&handleDoABarrelRoll, "doABarrelRoll", {false});
         config.registerHandler(&handleFlagsTest, "flags", {true});
         config.registerHandler(&handleSource, "source", {false});
+        config.registerHandler(&handleTestIgnoreKeyword, "testIgnoreKeyword", {false});
+        config.registerHandler(&handleTestUseKeyword, ":testUseKeyword", {false});
+        config.registerHandler(&handleNoop, "testCategory:testUseKeyword", {false});
+        config.registerHandler(&handleCategoryKeyword, "testCategory:categoryKeyword", {false});
 
         config.addSpecialCategory("special", {"key"});
         config.addSpecialConfigValue("special", "value", (Hyprlang::INT)0);
@@ -149,6 +180,20 @@ int main(int argc, char** argv, char** envp) {
         EXPECT(std::any_cast<int64_t>(config.getConfigValue("testCategory:testColor2")), (Hyprlang::INT)0xFF000000);
         EXPECT(std::any_cast<int64_t>(config.getConfigValue("testCategory:testColor3")), (Hyprlang::INT)0x22ffeeff);
         EXPECT(std::any_cast<const char*>(config.getConfigValue("testStringColon")), std::string{"ee:ee:ee"});
+        EXPECT(std::any_cast<const char*>(config.getConfigValue("categoryKeyword")), std::string{"oops, this one shouldn't call the handler, not fun"});
+        EXPECT(std::any_cast<const char*>(config.getConfigValue("testCategory:nested1:categoryKeyword")), std::string{"this one should not either"});
+        EXPECT(ignoreKeyword, "aaa");
+        EXPECT(useKeyword, "yes");
+
+        // Test templated wrapper
+        auto T1 = Hyprlang::CSimpleConfigValue<Hyprlang::INT>(&config, "testInt");
+        auto T2 = Hyprlang::CSimpleConfigValue<Hyprlang::FLOAT>(&config, "testFloat");
+        auto T3 = Hyprlang::CSimpleConfigValue<Hyprlang::SVector2D>(&config, "testVec");
+        auto T4 = Hyprlang::CSimpleConfigValue<std::string>(&config, "testString");
+        EXPECT(*T1, 123);
+        EXPECT(*T2, 123.456F);
+        EXPECT(*T3, EXP);
+        EXPECT(*T4, "Hello World! # This is not a comment!");
 
         // test static values
         std::cout << " → Testing static values\n";
@@ -162,6 +207,10 @@ int main(int argc, char** argv, char** envp) {
         EXPECT(barrelRoll, true);
         EXPECT(flagsFound, std::string{"abc"});
 
+        EXPECT(categoryKeywordActualValues.at(0), "we are having fun");
+        EXPECT(categoryKeywordActualValues.at(1), "so much fun");
+        EXPECT(categoryKeywordActualValues.at(2), "im the fun one at parties");
+
         // test dynamic
         std::cout << " → Testing dynamic\n";
         barrelRoll = false;
@@ -240,6 +289,24 @@ int main(int argc, char** argv, char** envp) {
         const auto ERRORSTR = std::string{ERRORS.getError()};
         EXPECT(std::count(ERRORSTR.begin(), ERRORSTR.end(), '\n'), 1);
 
+        std::cout << " → Testing invalid-numbers.conf\n";
+        Hyprlang::CConfig invalidNumbersConfig("./config/invalid-numbers.conf", {.throwAllErrors = true});
+        invalidNumbersConfig.addConfigValue("invalidHex", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("emptyHex", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("hugeHex", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("invalidInt", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("emptyInt", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("invalidColor", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("invalidFirstCharColor", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("invalidColorAlpha", (Hyprlang::INT)0);
+        invalidNumbersConfig.addConfigValue("invalidFirstCharColorAlpha", (Hyprlang::INT)0);
+
+        invalidNumbersConfig.commence();
+        const auto ERRORS2 = invalidNumbersConfig.parse();
+
+        EXPECT(ERRORS2.error, true);
+        const auto ERRORSTR2 = std::string{ERRORS2.getError()};
+        EXPECT(std::count(ERRORSTR2.begin(), ERRORSTR2.end(), '\n'), 9 - 1);
     } catch (const char* e) {
         std::cout << Colors::RED << "Error: " << Colors::RESET << e << "\n";
         return 1;
